import java.util.Arrays;
import java.util.Iterator;
import java.util.NoSuchElementException;
/** An instance contains an iterator over permutations of a sequence of integers.*/
public class Permutation implements Iterable<Integer[]> {
    
    /** Test the for-each loop over the permutations of {1, 2, 3, 4}*/
    public static void main(String[] args) {
        Integer[] c= {1, 2, 3, 4};
        Permutation p= new Permutation(c);
        System.out.println("Permutations of {1, 2, 3, 4}");
        for (Integer[] p1 : p) {
            System.out.println(Arrays.toString(p1));
        }
    }

    private Integer[] b; // The list of values. It is sorted.
    
    /** An instance for the list of values in b.
     * Precondition: b.length > 0 and b contains no duplicates  */
    public Permutation(Integer[] b) {
        // The array is copied because we don't want the user to be able to change
        // it! It is sorted so that it is always the smallest permutation.
        this.b= Arrays.copyOf(b, b.length);
        Arrays.sort(this.b);
    }
    
    /** Return an iterator over the list of values. */
    public Iterator<Integer[]> iterator() {
        return new PermutationIterator();
    }
    
    
    
    /** An instance enumerates the permutations of b */
    private class PermutationIterator implements Iterator<Integer[]> {
        // In order that all calls to next() and hasNext() wok properly, we use
        // a field ready. Its definition is given below.
        // Note that c contains a COPY of the array whose permutations are to
        // be generated. That's because the user might want to generate all
        // of the array two or three times at once (e.g. a foreach loop inside a
        // foreach loop), so array b, on line 17, cannot be used. When an instance
        // of PermutationIterator is called, that b is copied into field c.
        
        boolean ready= true; // true iff c contains the next permutation to generate
                             // false iff c contains the last permutation generated
        private Integer[] c;
        
        /** An instance that provides permutations of the original array. */
        public PermutationIterator() {
            // Note that this localb is sorted initially, and its length is > 1
            c= Arrays.copyOf(b, b.length);
            
        }
        
        /** = "There is another permutation to enumerate. */
        public @Override boolean hasNext() {
            if (ready) return true;
            ready= nextPerm();
            return ready;
        }
        
        /** = the next permutation to enumerate. */
        public @Override Integer[] next() {
            if (ready) {
                ready= false;
                return c;
            }
            // c has to be changed into the next permutation, but if
            // that is not possible throw a NoSuchElementException
            ready= nextPerm();
            if (ready) {
                ready= false;
                return c;
            }
            throw new NoSuchElementException();
        
        }
        
        /** Throw an UnsupportedOperationException - not supported. */
        public @Override void remove() {
            throw new UnsupportedOperationException();
        }
        

        
        /** Change c into the next permutation (if possible) and return
         * whether the change was made. */
        public boolean nextPerm() {
            // Store in k the largest int such that c[k] < c[k+1]
            // c[k] is the leftmost element of c to change.
            int k= c.length-2;
            while (k >= 0  &&  c[k].compareTo(c[k+1]) >= 0) {
                  k= k-1;
            }
            if (k == -1) return false;  // it's already the highest permutation
            
            // { c[k+1..] is nonempty and is an decreasing sequence! }
            // Find the rightmost j such that c[j] > c[k] --it exists
            int j= c.length-1;
            while (c[j] <= c[k]) {
                j= j-1;
            }
            
            // Swap c[k] and c[j]
            Integer t= c[k]; c[k]= c[j]; c[j]= t;
            
            // { c[k+1..] is still an decreasing sequence!}
            // It has to be changed into an increasing sequence so t is as small as possible
            
            // Reverse c[k+1..c.length-1]
            int m= k+1; int n= c.length-1;
            // invariant: c[m..n] remains to be reversed
            while (m < n) {
                Integer u= c[m]; c[m]= c[n]; c[n]= u;
                m= m+1; n= n-1;
            }
            
            return true;
        }
        
        
    }
}
